@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2020 CERN
+# Copyright 2012-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2020
 # - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
 # - Yun-Pin Sun <winter0128@gmail.com>, 2013
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2020
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2021
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2020
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2015
 # - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015-2018
@@ -34,7 +34,8 @@
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 '''
  Client class for callers of the Rucio system
@@ -46,7 +47,14 @@ try:
     import importlib
     importlib.util.find_spec('')
 except AttributeError:
+    pass
+
+# Fallback until Python <3.9
+# Must be removed with Python 3.9
+try:
     import imp
+except AttributeError:
+    pass
 
 import os
 import random
@@ -107,6 +115,20 @@ REGION = make_region(function_key_generator=my_key_generator).configure(
     expiration_time=60,
 )
 
+STATUS_CODES_TO_RETRY = [502, 503, 504]
+MAX_RETRY_BACK_OFF_SECONDS = 10
+
+
+def back_off(retry_number, reason):
+    """
+    Sleep a certain amount of time which increases with the retry count
+    :param retry_number: the retry iteration
+    :param reason: the reason to backoff which will be shown to the user
+    """
+    sleep_time = min(MAX_RETRY_BACK_OFF_SECONDS, 0.25 * 2 ** retry_number)
+    LOG.warning("Waiting {}s due to reason: {} ".format(sleep_time, reason))
+    time.sleep(sleep_time)
+
 
 @REGION.cache_on_arguments(namespace='host_to_choose')
 def choice(hosts):
@@ -395,6 +417,9 @@ class BaseClient(object):
                     result = self.session.delete(url, headers=hds, data=data, verify=self.ca_cert, timeout=self.timeout)
                 else:
                     return
+                if result.status_code in STATUS_CODES_TO_RETRY:
+                    back_off(retry, reason='server returned {}'.format(result.status_code))
+                    continue
             except ConnectionError as error:
                 LOG.error('ConnectionError: ' + str(error))
                 if retry > self.request_retries:
@@ -430,7 +455,9 @@ class BaseClient(object):
         for retry in range(self.AUTH_RETRIES + 1):
             try:
                 result = self.session.get(url, headers=headers, verify=self.ca_cert)
-                break
+                if result.status_code in STATUS_CODES_TO_RETRY:
+                    back_off(retry, reason='server returned {}'.format(result.status_code))
+                    continue
             except ConnectionError as error:
                 LOG.error('ConnectionError: ' + str(error))
                 if retry > self.request_retries:
@@ -514,6 +541,9 @@ class BaseClient(object):
                         LOG.debug("No new token was received, possibly invalid/expired \
                                    \ntoken or a token with no refresh token in Rucio DB")
                         return False
+                elif refresh_result.status_code in STATUS_CODES_TO_RETRY:
+                    back_off(retry, reason='server returned {}'.format(refresh_result.status_code))
+                    continue
                 else:
                     print("Rucio Client did not succeed to contact the \
                            \nRucio Auth Server when attempting token refresh.")
@@ -698,6 +728,9 @@ class BaseClient(object):
         for retry in range(self.AUTH_RETRIES + 1):
             try:
                 result = self.session.get(url, headers=headers, cert=cert, verify=self.ca_cert)
+                if result.status_code in STATUS_CODES_TO_RETRY:
+                    back_off(retry, reason='server returned {}'.format(result.status_code))
+                    continue
                 break
             except ConnectionError as error:
                 if 'alert certificate expired' in str(error):
@@ -743,6 +776,9 @@ class BaseClient(object):
         for retry in range(self.AUTH_RETRIES + 1):
             try:
                 result = self.session.get(url, headers=headers, verify=self.ca_cert)
+                if result.status_code in STATUS_CODES_TO_RETRY:
+                    back_off(retry, reason='server returned {}'.format(result.status_code))
+                    continue
                 break
             except ConnectionError as error:
                 if 'alert certificate expired' in str(error):
@@ -776,6 +812,9 @@ class BaseClient(object):
         for retry in range(self.AUTH_RETRIES + 1):
             try:
                 result = self.session.get(url, headers=headers, verify=self.ca_cert)
+                if result.status_code in STATUS_CODES_TO_RETRY:
+                    back_off(retry, reason='server returned {}'.format(result.status_code))
+                    continue
                 break
             except ConnectionError as error:
                 if 'alert certificate expired' in str(error):
@@ -814,6 +853,9 @@ class BaseClient(object):
             try:
                 result = self.session.get(url, headers=headers,
                                           verify=self.ca_cert, auth=HTTPKerberosAuth())
+                if result.status_code in STATUS_CODES_TO_RETRY:
+                    back_off(retry, reason='server returned {}'.format(result.status_code))
+                    continue
                 break
             except ConnectionError as error:
                 LOG.error('ConnectionError: ' + str(error))
@@ -848,6 +890,9 @@ class BaseClient(object):
         for retry in range(self.AUTH_RETRIES + 1):
             try:
                 SAML_auth_result = self.session.get(url, headers=headers)
+                if SAML_auth_result.status_code in STATUS_CODES_TO_RETRY:
+                    back_off(retry, reason='server returned {}'.format(SAML_auth_result.status_code))
+                    continue
 
                 if SAML_auth_result.headers['X-Rucio-Auth-Token']:
                     return SAML_auth_result.headers['X-Rucio-Auth-Token']
