@@ -62,14 +62,17 @@ from rucio import version
 
 class UploadClient:
 
-    def __init__(self, _client=None, logger=logging.log, tracing=True):
+    def __init__(self, _client=None, logger=None, tracing=True):
         """
         Initialises the basic settings for an UploadClient object
 
         :param _client:     - Optional: rucio.client.client.Client object. If None, a new object will be created.
-        :param logger:      - Optional: decorated logging.log object that can be passed from the calling daemon or client.
+        :param logger:      - Optional: logging.Logger object. If None, default logger will be used.
         """
-        self.logger = logger
+        if not logger:
+            self.logger = logging.log
+        else:
+            self.logger = logger.log
 
         self.client = _client if _client else Client()
         self.client_location = detect_client_location()
@@ -394,7 +397,7 @@ class UploadClient:
             logger(logging.INFO, 'File DID already exists')
             logger(logging.DEBUG, 'local checksum: %s, remote checksum: %s' % (file['adler32'], meta['adler32']))
 
-            if meta['adler32'] != file['adler32']:
+            if str(meta['adler32']).lstrip('0') != str(file['adler32']).lstrip('0'):
                 logger(logging.ERROR, 'Local checksum %s does not match remote checksum %s' % (file['adler32'], meta['adler32']))
 
                 raise DataIdentifierAlreadyExists
@@ -643,7 +646,7 @@ class UploadClient:
                 if rse_settings['verify_checksum'] is not False:
                     if ('adler32' in stats) and ('adler32' in lfn):
                         self.logger(logging.DEBUG, 'Checksum: Expected=%s Found=%s' % (lfn['adler32'], stats['adler32']))
-                        if stats['adler32'] != lfn['adler32']:
+                        if str(stats['adler32']).lstrip('0') != str(lfn['adler32']).lstrip('0'):
                             raise RucioException('Checksum mismatch. Source: %s Destination: %s' % (lfn['adler32'], stats['adler32']))
 
             except Exception as error:
